@@ -9,8 +9,10 @@ import codecs
 import ranger.api
 from ranger.core.linemode import LinemodeBase
 
+
 @ranger.api.register_linemode
 class MyLinemode(LinemodeBase):
     name = "rot13"
+
     def filetitle(self, file, metadata):
-        return codecs.encode(file.basename, "rot_13")
+        return codecs.encode(file.relative_path, "rot_13")
