@@ -6,6 +6,7 @@
 import os
 import ranger
 import re
+import inspect
 from collections import deque
 from ranger.api import *
 from ranger.core.shared import FileManagerAware
@@ -13,6 +14,7 @@ from ranger.ext.lazy_property import lazy_property
 
 _SETTINGS_RE = re.compile(r'^\s*([^\s]+?)=(.*)$')
 
+
 class CommandContainer(object):
     def __init__(self):
         self.commands = {}
@@ -29,7 +31,7 @@ class CommandContainer(object):
             cmd._line = full_command
             self.commands[name] = cmd
 
-        except:
+        except Exception:
             pass
 
     def load_commands_from_module(self, module):
@@ -47,7 +49,7 @@ class CommandContainer(object):
                 continue
             attribute = getattr(obj, attribute_name)
             if hasattr(attribute, '__call__'):
-                cmd = type(attribute_name, (FunctionCommand, ), dict())
+                cmd = type(attribute_name, (FunctionCommand, ), dict(__doc__=attribute.__doc__))
                 cmd._based_function = attribute
                 cmd._function_name = attribute.__name__
                 cmd._object_name = obj.__class__.__name__
@@ -55,8 +57,8 @@ class CommandContainer(object):
 
     def get_command(self, name, abbrev=True):
         if abbrev:
-            lst = [cls for cmd, cls in self.commands.items() \
-                    if cls.allow_abbrev and cmd.startswith(name) \
+            lst = [cls for cmd, cls in self.commands.items()
+                    if cls.allow_abbrev and cmd.startswith(name)
                     or cmd == name]
             if len(lst) == 0:
                 raise KeyError
@@ -106,7 +108,7 @@ class Command(FileManagerAware):
     def execute(self):
         """Override this"""
 
-    def tab(self):
+    def tab(self, tabnum):
         """Override this"""
 
     def quick(self):
@@ -140,7 +142,7 @@ class Command(FileManagerAware):
 
     def start(self, n):
         """Returns everything until (inclusively) arg(n)"""
-        return ' '.join(self.args[:n]) + " " # XXX
+        return ' '.join(self.args[:n]) + " "  # XXX
 
     def shift(self):
         del self.args[0]
@@ -151,6 +153,27 @@ class Command(FileManagerAware):
         return ''.join([self._tabinsert_left, word, self._tabinsert_right])
 
     def parse_setting_line(self):
+        """
+        Parses the command line argument that is passed to the `:set` command.
+        Returns [option, value, name_complete].
+
+        Can parse incomplete lines too, and `name_complete` is a boolean
+        indicating whether the option name looks like it's completed or
+        unfinished.  This is useful for generating tab completions.
+
+        >>> Command("set foo=bar").parse_setting_line()
+        ['foo', 'bar', True]
+        >>> Command("set foo").parse_setting_line()
+        ['foo', '', False]
+        >>> Command("set foo=").parse_setting_line()
+        ['foo', '', True]
+        >>> Command("set foo ").parse_setting_line()
+        ['foo', '', True]
+        >>> Command("set myoption myvalue").parse_setting_line()
+        ['myoption', 'myvalue', True]
+        >>> Command("set").parse_setting_line()
+        ['', '', False]
+        """
         if self._setting_line is not None:
             return self._setting_line
         match = _SETTINGS_RE.match(self.rest(1))
@@ -162,6 +185,25 @@ class Command(FileManagerAware):
         self._setting_line = result
         return result
 
+    def parse_setting_line_v2(self):
+        """
+        Parses the command line argument that is passed to the `:set` command.
+        Returns [option, value, name_complete, toggle].
+
+        >>> Command("set foo=bar").parse_setting_line_v2()
+        ['foo', 'bar', True, False]
+        >>> Command("set foo!").parse_setting_line_v2()
+        ['foo', '', True, True]
+        """
+        option, value, name_complete = self.parse_setting_line()
+        if len(option) >= 2 and option[-1] == '!':
+            toggle = True
+            option = option[:-1]
+            name_complete = True
+        else:
+            toggle = False
+        return [option, value, name_complete, toggle]
+
     def parse_flags(self):
         """Finds and returns flags in the command
 
@@ -232,7 +274,7 @@ class Command(FileManagerAware):
             # are we in the middle of the filename?
             else:
                 _, dirnames, _ = next(os.walk(abs_dirname))
-                dirnames = [dn for dn in dirnames \
+                dirnames = [dn for dn in dirnames
                         if dn.startswith(rel_basename)]
         except (OSError, StopIteration):
             # os.walk found nothing
@@ -291,7 +333,7 @@ class Command(FileManagerAware):
             else:
                 if directory.content_loaded:
                     # Take the order from the directory object
-                    names = [f.basename for f in directory.files \
+                    names = [f.basename for f in directory.files
                             if f.basename.startswith(rel_basename)]
                     if self.fm.thisfile.basename in names:
                         i = names.index(self.fm.thisfile.basename)
@@ -299,7 +341,7 @@ class Command(FileManagerAware):
                 else:
                     # Fall back to old method with "os.walk"
                     _, dirnames, filenames = next(os.walk(abs_dirname))
-                    names = [name for name in (dirnames + filenames) \
+                    names = [name for name in (dirnames + filenames)
                             if name.startswith(rel_basename)]
                     names.sort()
         except (OSError, StopIteration):
@@ -322,7 +364,7 @@ class Command(FileManagerAware):
 
     def _tab_through_executables(self):
         from ranger.ext.get_executables import get_executables
-        programs = [program for program in get_executables() if \
+        programs = [program for program in get_executables() if
                 program.startswith(self.rest(1))]
         if not programs:
             return
@@ -336,12 +378,13 @@ class FunctionCommand(Command):
     _based_function = None
     _object_name = ""
     _function_name = "unknown"
+
     def execute(self):
         if not self._based_function:
             return
         if len(self.args) == 1:
             try:
-                return self._based_function(**{'narg':self.quantifier})
+                return self._based_function(**{'narg': self.quantifier})
             except TypeError:
                 return self._based_function()
 
@@ -351,13 +394,13 @@ class FunctionCommand(Command):
             value = arg if (equal_sign is -1) else arg[equal_sign + 1:]
             try:
                 value = int(value)
-            except:
+            except Exception:
                 if value in ('True', 'False'):
                     value = (value == 'True')
                 else:
                     try:
                         value = float(value)
-                    except:
+                    except Exception:
                         pass
 
             if equal_sign == -1:
@@ -385,19 +428,24 @@ class FunctionCommand(Command):
                         (self._object_name, self._function_name,
                             repr(args), repr(keywords)), bad=True)
 
+
 class AliasCommand(Command):
     _based_function = None
     _object_name = ""
     _function_name = "unknown"
     _line = ""
+
     def execute(self):
         return self._make_cmd().execute()
 
     def quick(self):
         return self._make_cmd().quick()
 
-    def tab(self):
-        return self._make_cmd().tab()
+    def tab(self, tabnum):
+        cmd = self._make_cmd()
+        args = inspect.signature(cmd.tab).parameters if self.fm.py3 else \
+            inspect.getargspec(cmd.tab).args
+        return cmd.tab(tabnum) if 'tabnum' in args else cmd.tab()
 
     def cancel(self):
         return self._make_cmd().cancel()
