@@ -15,6 +15,7 @@ import sys
 import ranger.api
 from ranger.core.actions import Actions
 from ranger.core.tab import Tab
+from ranger.container import settings
 from ranger.container.tags import Tags, TagsDummy
 from ranger.gui.ui import UI
 from ranger.container.bookmarks import Bookmarks
@@ -27,6 +28,7 @@ from ranger.ext.signals import SignalDispatcher
 from ranger import __version__
 from ranger.core.loader import Loader
 
+
 class FM(Actions, SignalDispatcher):
     input_blocked = False
     input_blocked_until = 0
@@ -48,7 +50,7 @@ class FM(Actions, SignalDispatcher):
             self.ui = ui
         self.start_paths = paths
         self.directories = dict()
-        self.log = deque(maxlen=20)
+        self.log = deque(maxlen=1000)
         self.bookmarks = bookmarks
         self.current_tab = 1
         self.tabs = {}
@@ -64,14 +66,14 @@ class FM(Actions, SignalDispatcher):
 
         try:
             self.username = pwd.getpwuid(os.geteuid()).pw_name
-        except:
+        except Exception:
             self.username = 'uid:' + str(os.geteuid())
         self.hostname = socket.gethostname()
         self.home_path = os.path.expanduser('~')
 
-        self.log.append('ranger {0} started! Process ID is {1}.' \
+        self.log.appendleft('ranger {0} started! Process ID is {1}.'
                 .format(__version__, os.getpid()))
-        self.log.append('Running on Python ' + sys.version.replace('\n',''))
+        self.log.appendleft('Running on Python ' + sys.version.replace('\n', ''))
 
         mimetypes.knownfiles.append(os.path.expanduser('~/.mime.types'))
         mimetypes.knownfiles.append(self.relpath('data/mime.types'))
@@ -80,7 +82,7 @@ class FM(Actions, SignalDispatcher):
     def initialize(self):
         """If ui/bookmarks are None, they will be initialized here."""
 
-        self.tabs = dict((n+1, Tab(path)) for n, path in
+        self.tabs = dict((n + 1, Tab(path)) for n, path in
                 enumerate(self.start_paths))
         tab_list = self._get_tab_list()
         if tab_list:
@@ -96,7 +98,13 @@ class FM(Actions, SignalDispatcher):
             rifleconf = self.relpath('config/rifle.conf')
         self.rifle = Rifle(rifleconf)
         self.rifle.reload_config()
-        self.image_displayer = self._get_image_displayer()
+
+        def set_image_displayer():
+            self.image_displayer = self._get_image_displayer()
+        set_image_displayer()
+        self.settings.signal_bind('setopt.preview_images_method',
+                set_image_displayer,
+                priority=settings.SIGNAL_PRIORITY_AFTER_SYNC)
 
         if not ranger.arg.clean and self.tags is None:
             self.tags = Tags(self.confpath('tagged'))
@@ -136,25 +144,36 @@ class FM(Actions, SignalDispatcher):
 
             if self.settings.open_all_images and \
                     len(self.thisdir.marked_items) == 0 and \
-                    re.match(r'^(feh|sxiv) ', command):
+                    re.match(r'^(feh|sxiv|imv|pqiv) ', command):
 
-                images = [f.basename for f in self.thisdir.files if f.image]
-                escaped_filenames = " ".join(shell_quote(f) \
+                images = [f.relative_path for f in self.thisdir.files if f.image]
+                escaped_filenames = " ".join(shell_quote(f)
                         for f in images if "\x00" not in f)
 
-                if images and self.thisfile.basename in images and \
+                if images and self.thisfile.relative_path in images and \
                         "$@" in command:
                     new_command = None
 
                     if command[0:5] == 'sxiv ':
-                        number = images.index(self.thisfile.basename) + 1
+                        number = images.index(self.thisfile.relative_path) + 1
                         new_command = command.replace("sxiv ",
                                 "sxiv -n %d " % number, 1)
 
                     if command[0:4] == 'feh ':
                         new_command = command.replace("feh ",
-                            "feh --start-at %s " % \
-                            shell_quote(self.thisfile.basename), 1)
+                            "feh --start-at %s " %
+                            shell_quote(self.thisfile.relative_path), 1)
+
+                    if command[0:4] == 'imv ':
+                        number = images.index(self.thisfile.relative_path) + 1
+                        new_command = command.replace("imv ",
+                                "imv -n %d " % number, 1)
+
+                    if command[0:5] == 'pqiv ':
+                        number = images.index(self.thisfile.relative_path)
+                        new_command = command.replace("pqiv ",
+                                "pqiv --action \"goto_file_byindex(%d)\" " %
+                                number, 1)
 
                     if new_command:
                         command = "set -- %s; %s" % (escaped_filenames,
@@ -176,13 +195,13 @@ class FM(Actions, SignalDispatcher):
         if self.ui:
             try:
                 self.ui.destroy()
-            except:
+            except Exception:
                 if debug:
                     raise
         if self.loader:
             try:
                 self.loader.destroy()
-            except:
+            except Exception:
                 if debug:
                     raise
 
@@ -191,6 +210,10 @@ class FM(Actions, SignalDispatcher):
             return W3MImageDisplayer()
         elif self.settings.preview_images_method == "iterm2":
             return ITerm2ImageDisplayer()
+        elif self.settings.preview_images_method == "urxvt":
+            return URXVTImageDisplayer()
+        elif self.settings.preview_images_method == "urxvt-full":
+            return URXVTImageFSDisplayer()
         else:
             return ImageDisplayer()
 
@@ -207,7 +230,7 @@ class FM(Actions, SignalDispatcher):
         self.thistab.thisdir = obj
 
     thisfile = property(_get_thisfile, _set_thisfile)
-    thisdir  = property(_get_thisdir,  _set_thisdir)
+    thisdir  = property(_get_thisdir, _set_thisdir)
 
     def block_input(self, sec=0):
         self.input_blocked = sec != 0
@@ -224,6 +247,7 @@ class FM(Actions, SignalDispatcher):
             return
         import shutil
         from errno import EEXIST
+
         def copy(_from, to):
             if os.path.exists(self.confpath(to)):
                 sys.stderr.write("already exists: %s\n" % self.confpath(to))
