@@ -1,14 +1,16 @@
-# Copyright (C) 2009-2013  Roman Zimbelmann <hut@lavabit.com>
-# This software is distributed under the terms of the GNU GPL version 3.
+# This file is part of ranger, the console file manager.
+# License: GNU GPL version 3, see the file "AUTHORS" for details.
 
 import os
 import sys
 from os.path import abspath, normpath, join, expanduser, isdir
 
+from ranger.container import settings
 from ranger.container.history import History
 from ranger.core.shared import FileManagerAware, SettingsAware
 from ranger.ext.signals import SignalDispatcher
 
+
 class Tab(FileManagerAware, SettingsAware):
     def __init__(self, path):
         self.thisdir = None  # Current Working Directory
@@ -21,10 +23,11 @@ class Tab(FileManagerAware, SettingsAware):
         # NOTE: in the line below, weak=True works only in python3.  In python2,
         # weak references are not equal to the original object when tested with
         # "==", and this breaks _set_thisfile_from_signal and _on_tab_change.
-        self.fm.signal_bind('move', self._set_thisfile_from_signal, priority=0.1,
-                weak=(sys.version > '3'))
+        self.fm.signal_bind('move', self._set_thisfile_from_signal,
+                priority=settings.SIGNAL_PRIORITY_AFTER_SYNC,
+                weak=(sys.version_info[0] >= 3))
         self.fm.signal_bind('tab.change', self._on_tab_change,
-                weak=(sys.version > '3'))
+                weak=(sys.version_info[0] >= 3))
 
     def _set_thisfile_from_signal(self, signal):
         if self == signal.tab:
@@ -102,12 +105,18 @@ class Tab(FileManagerAware, SettingsAware):
     def inherit_history(self, other_history):
         self.history.rebase(other_history)
 
-    def enter_dir(self, path, history = True):
+    def enter_dir(self, path, history=True):
         """Enter given path"""
         # TODO: Ensure that there is always a self.thisdir
-        if path is None: return
+        if path is None:
+            return
         path = str(path)
 
+        # clear filter in the folder we're leaving
+        if self.fm.settings.clear_filters_on_dir_change and self.thisdir:
+            self.thisdir.filter = None
+            self.thisdir.refilter()
+
         previous = self.thisdir
 
         # get the absolute path
@@ -119,7 +128,7 @@ class Tab(FileManagerAware, SettingsAware):
 
         try:
             os.chdir(path)
-        except:
+        except Exception:
             return True
         self.path = path
         self.thisdir = new_thisdir
