@@ -4,6 +4,7 @@
 from stat import S_IXOTH, S_IFREG
 from ranger.ext.iter_tools import unique
 from os import listdir, environ, stat
+import shlex
 
 
 _cached_executables = None
@@ -33,15 +34,27 @@ def get_executables_uncached(*paths):
     for path in paths:
         try:
             content = listdir(path)
-        except:
+        except Exception:
             continue
         for item in content:
             abspath = path + '/' + item
             try:
                 filestat = stat(abspath)
-            except:
+            except Exception:
                 continue
             if filestat.st_mode & (S_IXOTH | S_IFREG):
                 executables.add(item)
     return executables
 
+
+def get_term():
+    """Get the user terminal executable name.
+
+    Either $TERMCMD, $TERM, "x-terminal-emulator" or "xterm", in this order.
+    """
+    command = environ.get('TERMCMD', environ.get('TERM'))
+    if shlex.split(command)[0] not in get_executables():
+        command = 'x-terminal-emulator'
+        if command not in get_executables():
+            command = 'xterm'
+    return command
