@@ -1,6 +1,6 @@
-# Copyright (C) 2010 David Barnett <davidbarnett2@gmail.com>
-# Copyright (C) 2010-2013  Roman Zimbelmann <hut@lavabit.com>
-# This software is distributed under the terms of the GNU GPL version 3.
+# This file is part of ranger, the console file manager.
+# License: GNU GPL version 3, see the file "AUTHORS" for details.
+# Author: David Barnett <davidbarnett2@gmail.com>, 2010
 
 """A library to help to convert ANSI codes to curses instructions."""
 
@@ -8,14 +8,17 @@ from ranger.gui import color
 import re
 
 ansi_re = re.compile('(\x1b' + r'\[\d*(?:;\d+)*?[a-zA-Z])')
-codesplit_re = re.compile('38;5;(\d+);|48;5;(\d+);|(\d*);')
+codesplit_re = re.compile(r'38;5;(\d+);|48;5;(\d+);|(\d*);')
 reset = '\x1b[0m'
 
+
 def split_ansi_from_text(ansi_text):
     return ansi_re.split(ansi_text)
 
 # For information on the ANSI codes see
 # githttp://en.wikipedia.org/wiki/ANSI_escape_code
+
+
 def text_with_fg_bg_attr(ansi_text):
     fg, bg, attr = -1, -1, 0
     for chunk in split_ansi_from_text(ansi_text):
@@ -42,7 +45,7 @@ def text_with_fg_bg_attr(ansi_text):
                         n = int(arg)
                     else:                         # empty code means reset
                         n = 0
-                except:
+                except Exception:
                     continue
 
                 if n == 0:                        # reset colors and attributes
@@ -93,6 +96,7 @@ def text_with_fg_bg_attr(ansi_text):
         else:
             yield chunk
 
+
 def char_len(ansi_text):
     """Count the number of visible characters.
 
@@ -109,6 +113,7 @@ def char_len(ansi_text):
     """
     return len(ansi_re.sub('', ansi_text))
 
+
 def char_slice(ansi_text, start, length):
     """Slices a string with respect to ansi code sequences
 
@@ -144,13 +149,13 @@ def char_slice(ansi_text, start, length):
         old_pos = pos
         pos += len(chunk)
         if pos <= start:
-            pass # seek
+            pass  # seek
         elif old_pos < start and pos >= start:
             chunks.append(last_color)
-            chunks.append(chunk[start-old_pos:start-old_pos+length])
+            chunks.append(chunk[start - old_pos:start - old_pos + length])
         elif pos > length + start:
             chunks.append(last_color)
-            chunks.append(chunk[:start-old_pos+length])
+            chunks.append(chunk[:start - old_pos + length])
         else:
             chunks.append(last_color)
             chunks.append(chunk)
