@@ -10,6 +10,8 @@ from ranger.ext.direction import Direction
 from ranger.ext.img_display import ImgDisplayUnsupportedException
 
 # TODO: Scrolling in embedded pager
+
+
 class Pager(Widget):
     source = None
     source_is_stream = False
@@ -20,6 +22,7 @@ class Pager(Widget):
     need_clear_image = False
     need_redraw_image = False
     max_width = None
+
     def __init__(self, win, embedded=False):
         Widget.__init__(self, win)
         self.embedded = embedded
@@ -52,6 +55,7 @@ class Pager(Widget):
 
     def destroy(self):
         self.clear_image(force=True)
+        Widget.destroy(self)
 
     def finalize(self):
         self.fm.ui.win.move(self.y, self.x)
@@ -104,7 +108,7 @@ class Pager(Widget):
         elif self.markup == 'ansi':
             try:
                 self.win.move(i, 0)
-            except:
+            except Exception:
                 pass
             else:
                 for chunk in ansi.text_with_fg_bg_attr(line):
@@ -124,15 +128,24 @@ class Pager(Widget):
                     pagesize=self.wid,
                     offset=-self.wid + 1)
         if direction.vertical():
-            if self.source_is_stream:
-                self._get_line(self.scroll_begin + self.hei * 2)
-            self.scroll_begin = direction.move(
+            movement = dict(
                     direction=direction.down(),
                     override=narg,
-                    maximum=len(self.lines),
                     current=self.scroll_begin,
                     pagesize=self.hei,
                     offset=-self.hei + 1)
+            if self.source_is_stream:
+                # For streams, we first pretend that the content ends much later,
+                # in case there are still unread lines.
+                desired_position = direction.move(
+                        maximum=len(self.lines) + 9999,
+                        **movement)
+                # Then, read the new lines as needed to produce a more accurate
+                # maximum for the movement:
+                self._get_line(desired_position + self.hei)
+            self.scroll_begin = direction.move(
+                    maximum=len(self.lines),
+                    **movement)
 
     def press(self, key):
         self.fm.ui.keymaps.use_keymap('pager')
@@ -218,7 +231,7 @@ class Pager(Widget):
                     line = ansi.char_slice(line, startx, self.wid) + ansi.reset
                 else:
                     line = line[startx:self.wid + startx]
-                yield line.rstrip()
+                yield line.rstrip().replace('\r\n', '\n')
             except IndexError:
                 raise StopIteration
             i += 1
