@@ -1,5 +1,5 @@
-# Copyright (C) 2009-2013  Roman Zimbelmann <hut@lavabit.com>
-# This software is distributed under the terms of the GNU GPL version 3.
+# This file is part of ranger, the console file manager.
+# License: GNU GPL version 3, see the file "AUTHORS" for details.
 
 """The titlebar is the widget at the top, giving you broad overview.
 
@@ -8,9 +8,11 @@ It displays the current path among other things.
 
 from os.path import basename
 
+
 from . import Widget
 from ranger.gui.bar import Bar
 
+
 class TitleBar(Widget):
     old_thisfile = None
     old_keybuffer = None
@@ -73,7 +75,7 @@ class TitleBar(Widget):
                 else:
                     try:
                         self.fm.enter_dir(part.directory)
-                    except:
+                    except Exception:
                         pass
                 return True
         return False
@@ -98,12 +100,12 @@ class TitleBar(Widget):
         bar.add(self.fm.username, 'hostname', clr, fixed=True)
         bar.add('@', 'hostname', clr, fixed=True)
         bar.add(self.fm.hostname, 'hostname', clr, fixed=True)
-        bar.add(':', 'hostname', clr, fixed=True)
+        bar.add(' ', 'hostname', clr, fixed=True)
 
         pathway = self.fm.thistab.pathway
         if self.settings.tilde_in_titlebar and \
                 self.fm.thisdir.path.startswith(self.fm.home_path):
-            pathway = pathway[self.fm.home_path.count('/')+1:]
+            pathway = pathway[self.fm.home_path.count('/') + 1:]
             bar.add('~/', 'directory', fixed=True)
 
         for path in pathway:
@@ -115,8 +117,9 @@ class TitleBar(Widget):
             bar.add(path.basename, clr, directory=path)
             bar.add('/', clr, fixed=True, directory=path)
 
-        if self.fm.thisfile is not None:
-            bar.add(self.fm.thisfile.basename, 'file')
+        if self.fm.thisfile is not None and \
+                self.settings.show_selection_in_titlebar:
+            bar.add(self.fm.thisfile.relative_path, 'file')
 
     def _get_right_part(self, bar):
         # TODO: fix that pressed keys are cut off when chaining CTRL keys
@@ -139,7 +142,7 @@ class TitleBar(Widget):
             if not dirname:
                 result += ":/"
             elif len(dirname) > 15:
-                result += ":" + dirname[:14] + "~"
+                result += ":" + dirname[:14] + self.ellipsis[self.settings.unicode_ellipsis]
             else:
                 result += ":" + dirname
         return result
